# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Simple Login
# Copyright (C) 2016 byteShaft
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from simple_login.views.api import (
    ActivationAPIView,
    ActivationKeyRequestAPIView,
    LoginAPIView,
    PasswordResetRequestAPIView,
    PasswordChangeAPIView,
    StatusAPIView,
    RetrieveUpdateDestroyProfileAPIView,
    RegisterAPIView,
    TwitterLoginAPIView,
    FacebookLoginAPIView,
)

__all__ = [
    'ActivationAPIView',
    'ActivationKeyRequestAPIView',
    'LoginAPIView',
    'PasswordResetRequestAPIView',
    'PasswordChangeAPIView',
    'StatusAPIView',
    'RetrieveUpdateDestroyProfileAPIView',
    'RegisterAPIView',
    'TwitterLoginAPIView',
    'FacebookLoginAPIView'
]
